<?php namespace EwCacheInvalidator;

use EwCacheInvalidator\LedgeInvalidator;
use EwCacheInvalidator\PageSettings;

/**
 * Class Lcc
 * @package EwCacheInvalidator
 */
class Lcc {

    private $base_url;
    private $username;
    private $password;

    /**
     * Lcc constructor.
     */
    public function __construct() {

        $this->base_url = PageSettings::getQueryStreamerUrl();
        $this->username = PageSettings::getQueryStreamerUsername();
        $this->password = PageSettings::getQueryStreamerPassword();
    }

    /**
     * Connect to LCC.
     *
     * @return bool
     */
    public function connect() {
        return true;
    }

    /**
     * Register a listener.
     *
     * @return string
     */
    public function registerListener($query) {

        $query = trim($query . ' ' . PageSettings::getQuerySuffix());
        $json_data = [
            'query' => [
                'query_string' => [
                    'query' => $query
                ]
            ],
            'destination' => [
                'type' => 'cache',
                'url' => plugins_url('ew-cache-invalidator/src/push.php')
            ],
            'meta' => [
                'msg' => 'Automated query stream from Everyware ew-cache-invalidator plugin.'
            ],
            'config' => [
                'notifyOnNoLongerMatchingQuery' => true
            ]
        ];

        $response = wp_remote_post($this->base_url . 'streams', [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username. ':' . $this->password),
                'Content-Type' => 'application/json'
            ],
            'body' => json_encode($json_data)
        ]);

        if(!is_wp_error($response) && $response['response']['code'] === 200) {

            $response_json = json_decode($response['body']);

            if(isset($response_json->streamId)) {
                return $response_json->streamId;
            }
        }

        return false;
    }

    /**
     * Deregister a listener.
     *
     * @param $streamId
     * @return bool
     */
    public function deregisterListener($streamId) {

        $response = wp_remote_request($this->base_url . 'streams/' . $streamId, [
            'method' => 'DELETE',
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password)
            ]
        ]);

        if(!is_wp_error($response) && $response['response']['code'] === 200) {
            return true;
        }

        return false;
    }

    /**
     * Get connection status.
     *
     * @return bool
     */
    public function getStatus() {

        $response = wp_remote_get($this->base_url . 'streams?url=test&type=cache', [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode($this->username . ':' . $this->password)
            ]
        ]);

        if(!is_wp_error($response) && $response['response']['code'] === 200) {
            return true;
        }

        return false;
    }
}