<?php namespace EwCacheInvalidator;

use EwCacheInvalidator\PageDebug;
use EwCacheInvalidator\PageSettings;
use EwCacheInvalidator\EventHandler;

/**
 * Class Startup
 * @package EwCacheInvalidator
 */
class Startup {

    /**
     * Startup constructor.
     * @param $plugin_path
     */
    public function __construct($plugin_path) {

        $dependencyCheck = $this->checkPluginDependencies();
        if($dependencyCheck !== true) {

            add_action('admin_init', function() {
                deactivate_plugins('ew-cache-invalidator/bootstrap.php');
            });

            add_action('admin_notices', function() use ($dependencyCheck) {
                print '<div class="notice notice-error"><p>' . $dependencyCheck . '</p></div>';
            });
        }

        // Register pages for menu.
        add_action('admin_menu', function() {

            $page_debug = new PageDebug();
            $page_settings = new PageSettings();

            add_menu_page(__('EW cache invalidator', 'ew-cache-invalidator'), 'EW Cache', 'manage_options', 'ew-cache-invalidator', '', 'dashicons-admin-settings');
            add_submenu_page( 'ew-cache-invalidator', __('Cache invalidation settings', 'ew-cache-invalidator'), __('Settings', 'ew-cache-invalidator'), 'manage_options', 'ew-cache-invalidator', array( $page_settings, 'renderAdminPage' ) );
            add_submenu_page( 'ew-cache-invalidator', __('Cache invalidation data', 'ew-cache-invalidator'), __('Debug data', 'ew-cache-invalidator'), 'manage_options', 'ew-cache-invalidator-debug', array($page_debug, 'renderAdminPage' ) );
        });

        $this->setupAjax();
        if(PageSettings::checkQueryEventsActive()) {
            new EventHandler();
        }
    }

    private function checkPluginDependencies() {

        if(!defined('EVERY_VERSION')) {
            return '<strong>Everyware Cache Invalidator:</strong> This plugin needs Everyware to be active.';
        }

        if(!defined('EVERYBOARD_VERSION')) {
            return '<strong>Everyware Cache Invalidator:</strong> This plugin needs Everyboard to be active.';
        }

        if(!version_compare(EVERY_VERSION, EW_CACHE_EVERYWARE_MIN_VERSION, '>=')) {
            return '<strong>Everyware Cache Invalidator:</strong> Everyware version not up to date with version required by Everyware cache invalidator plugin.';
        }

        if(!version_compare(EVERYBOARD_VERSION, EW_CACHE_EVERYBOARD_MIN_VERSION, '>=')) {
            return '<strong>Everyware Cache Invalidator:</strong> Everyware version not up to date with version required by Everyware cache invalidator plugin.';
        }

        return true;
    }

    /**
     * Setup ajax functions.
     */
    private function setupAjax() {

        add_action('wp_ajax_ew_cache_invalidator_save_query_streamer', function() {
            PageSettings::ajaxSaveQueryStream();
        });

        add_action('wp_ajax_ew_cache_invalidator_save_query_suffix', function() {
            PageSettings::ajaxSaveQuerySuffix();
        });

        add_action('wp_ajax_ew_cache_invalidator_reset_all_queries', function() {
            PageSettings::ajaxResetAllQueries();
        });

        add_action('wp_ajax_ew_cache_invalidator_delete_all_queries', function() {
            PageSettings::ajaxDeleteAllQueries();
        });

        add_action('wp_ajax_ew_cache_invalidator_check_endpoint_status', function() {
           PageSettings::ajaxCheckEndpointStatus();
        });

        add_action('wp_ajax_ew_cache_invalidator_check_query_streamer_status', function() {
            PageSettings::ajaxCheckQueryStreamStatus();
        });
    }
}